sample_height = 15;		# Height of sample boxes
sample_width = 40;		# Width of sample boxes
sample_space = 5;		# Space between boxes
sample_text_x = 37;		# Where to start the right justified text.
num_ranges = 10;		# Number of ranges to break histogram into.
outfilename$ = "c:/user/dwilliss/biomass.rvc";

class ColorMap cmap;
class Color black, white;
class XmDrawingArea da;
class XmForm form;			# The main form dialog
class GraphicsContext gc;	# The context for drawing into the view
raster Composite, BiomassRaster, FilteredBiomassRaster;
vector BiomassVector;
class view view;
class group group;
class RasterLayer layer, overlay;			# Our base layer
class VectorLayer voverlay;
class MdispRegionTool tool;
class tooltip tip;
region reg;
array histo[256];
numeric cellsize, unitconv;
numeric doingtest;
numeric haveBiomassRaster, haveBiomassVector;
class XmToggleButton drawhisto;
class PushButtonItem buttonConvertToVector, buttonFilterVector;
str$ = "";
unitconv = GetUnitConvArea("sq meters", "acres");

proc setTip () {
	if (!haveBiomassRaster) {
		if (tool.HasPosition) {
			tip.string = "Press right button\nto close polygon";
			}
		else {
			tip.string = "Press the left button\nand draw around a field.";
			}
		}
	else {
		tip.string = "";
		}
	}

proc cbQuit(class widget widget) {
	DialogClose(form);
	}

proc drawGraph(class XmDrawingArea da) {
	local numeric maxval, total, i, scale, w, y;

	ActivateGC(gc);
	SetColorRGB(0,0,0);
	FillRect(0, 0, da.width, da.height);
	maxval = 0;
	total = 0;
	for i = 1 to num_ranges {
		maxval = SetMax(maxval, histo[i]);
		total += histo[i];
		}
	if (total) {	# Don't draw bars if nothing selected
		w = da.width - 2 * sample_space;
		scale = w / maxval;
		if (drawhisto.set) {
			for i = 1 to num_ranges {
				y = (num_ranges - i + 1) * (sample_height + sample_space) + sample_space;
				SetColor(ColorMapGetColor(cmap, i));
				w = scale * histo[i];
				if (w < 1 && histo[i] != 0) w = 1;
				FillRect(sample_space, y, w, sample_height);
				}
			}
		else {
			DrawTextSetColors(white, black);
			y = sample_space;
			gc.TextStyle.JustifyRight = 0;
			gc.TextStyle.JustifyCenter = 1;
			DrawTextSimple("Acres", da.width / 2, y + sample_height - 4);
			gc.TextStyle.JustifyRight = 1;
			gc.TextStyle.JustifyCenter = 0;
			DrawTextSetColors(black, white);
			for i = 1 to num_ranges {
				y = (num_ranges - i + 1) * (sample_height + sample_space) + sample_space;
				SetColor(ColorMapGetColor(cmap, i));
				FillRect(sample_space, y, sample_width, sample_height);
				SetColorName("white");
				DrawRect(sample_space, y, sample_width, sample_height);
				str$ = sprintf("%.2f", histo[i] * cellsize * unitconv);
				DrawTextSimple(str$, sample_text_x, y + sample_height - 4);
				}
			y = (num_ranges + 1) * (sample_height + sample_space) + sample_space;
			str$ = sprintf("%.2f", total * cellsize * unitconv);
			DrawTextSimple(str$, sample_text_x, y + sample_height - 4);
			}
		}
	}


proc cbToolApply(class RegionTool tool) {
	local numeric i, val, scale, minval, maxval, ir, red, total, offset, lo, hi;

	reg = tool.Region;
	FillRegion(RegionTrans(reg, ViewGetTransViewToScreen(view)));
	reg = RegionTrans(reg, ViewGetTransLayerToView(view, layer, 1));
	StatusSetDefaultHandle(view.StatusHandle);
	ViewSetMessage(view, "Filling raster with 0");
	BiomassRaster = 0;
	ViewSetMessage(view, "Computing scale");
	for i=0 to 200 histo[i] = 0;
	for each Composite in reg {
		ir = Composite.red;
		red = Composite.green;
		val = ((ir - red) / (ir + red) + 1) * 100;
		histo[val] = histo[val] + 1;
		}
	total = 0;
	for i=0 to 200 total += histo[i];
	lo = total * .02;
	hi = total * .98;
	minval = -100;
	maxval = -100;
	total = 0;
	for i = 0 to 200 {
		if (total > lo && minval < -1) {
			minval = (i / 100) - 1;
			}
		total += histo[i];
		if (total > hi && maxval < -1) {
			maxval = (i / 100) - 1;
			}
		}
	scale = num_ranges / (maxval - minval);
	offset = -minval;
	printf("offset = %f, scale = %f\n", offset, scale);

	ViewSetMessage(view, "Processing region");
	for i = 1 to num_ranges histo[i] = 0;
	for each Composite in reg {
		ir = Composite.red;
		red = Composite.green;
		val = (ir - red) / (ir + red);
		val = Bound((val + offset) * scale + 1, 1, num_ranges);
		histo[val] = histo[val] + 1;
		BiomassRaster = val;
		}
	CloseRaster(BiomassRaster);
	LayerShow(overlay, view);
	ViewRedraw(view);
	drawGraph(da);
	haveBiomassRaster = 1;
	tool.HasPosition = 0;	# Clears the current position
	buttonConvertToVector.Disabled = 0;
	setTip();
	}


proc cbConvToVector () {
	StatusSetDefaultHandle(view.StatusHandle);

	FilteredBiomassRaster = 0;
	ViewSetMessage(view, "Median filter");
	foreach BiomassRaster in reg {
		FilteredBiomassRaster = FocalMedian(BiomassRaster, 3, 3);
		}
	ViewSetMessage(view, "Converting Raster to Vector");
	BiomassVector = RasterToVectorBound(FilteredBiomassRaster);
	CloseVector(BiomassVector);
	if (!voverlay) {
		voverlay = GroupQuickAddVectorVar(group, BiomassVector);
		voverlay.NormalPolyStyle.BorderColor.name = "black";
		voverlay.NormalLineStyle.Color.name = "black";
		}
	LayerShow(voverlay, view);
	ViewRedraw(view);
	buttonFilterVector.Disabled = 0;
	}

proc cbFilterVector () {
	local i, numPolys, numToDelete;

	FilteredBiomassRaster = 0;
	StatusSetDefaultHandle(view.StatusHandle);
	ViewSetMessage(view, "Generating std stats table");
	VectorToolkitInit(BiomassVector);
	VectorUpdateStdAttributes(BiomassVector);		# Compute std stats table
	numToDelete = 0;
	numPolys = NumVectorPolys(BiomassVector);
	array deleteList[numPolys];
	ViewSetMessage(view, "Looking for polygons to delete");
	for i=1 to numPolys {
		if (BiomassVector.poly[i].POLYSTATS.Area < 40) {
			numToDelete += 1;
			deleteList[numToDelete] = i;
			}
		}
	if (numToDelete) {
		printf("Found %d polygons to delete\n", numToDelete);
		ViewSetMessage(view, "Deleting tiny polygons");
		VectorSetFlags(BiomassVector, "NoDBStatTable");
		VectorDeleteStdAttributes(BiomassVector);	
		VectorDeletePolys(BiomassVector, deleteList, numToDelete);
		}

	ViewSetMessage(view, "Closing vector");
	CloseVector(BiomassVector);
	ViewRedraw(view);
	}

proc cbHideOverlay(class widget widget) {
	local numeric i;
	LayerHide(overlay, view);
	for i = 0 to num_ranges histo[i] = 0;
	ViewRedraw(view);
	drawGraph(da);
	}

proc cbRedrawGraph() {
	drawGraph(da);
	}

proc cbToolPosSet(class tool tool) {
	setTip();
	}

GetInputRaster(Composite);
DeleteFile(outfilename$);
CreateTempRaster(BiomassRaster, NumLins(Composite), NumCols(Composite), "8-bit unsigned");
CopySubobjects(Composite, BiomassRaster, "georef");
cellsize = LinScale(Composite) * ColScale(Composite); 
c0 = ColorMapGetColor(cmap, 0);
c0.transp = 50;		# Make it 50% transparent
ColorMapSetColorHIS(cmap, 10, 230, 50, 100);	# Green
ColorMapSetColorHIS(cmap, 9, 180, 70, 100);		# Yellow
ColorMapSetColorHIS(cmap, 8, 150, 70, 100);		# Lt Orange
ColorMapSetColorHIS(cmap, 7, 144, 50, 100);		# Dk Orange
ColorMapSetColorHIS(cmap, 6, 120, 50, 100);		# Red
#ColorMapSetColorHIS(cmap, 5, 96, 50, 100);	
ColorMapSetColorHIS(cmap, 5, 72, 50, 100);		# Magenta
ColorMapSetColorHIS(cmap, 4, 48, 50, 100);		# Purple
ColorMapSetColorHIS(cmap, 3, 10, 50, 100);		# Blue
ColorMapSetColorHIS(cmap, 2, 342, 65, 100);		# Lt Blue
ColorMapSetColorHIS(cmap, 1, 320, 70, 100);		# Cyan
ColorMapSetColor(cmap, 0, c0);
ColorMapWriteToRastVar(BiomassRaster, cmap, "ColorMap");
SetNull(BiomassRaster, 0);

CreateTempRaster(FilteredBiomassRaster, NumLins(Composite), NumCols(Composite), "8-bit unsigned");
CopySubobjects(Composite, FilteredBiomassRaster, "georef");
CreateTempVector(BiomassVector);

form = CreateModalFormDialog("Relative Biomass Mapping");
buttonClearOverlay = CreatePushButtonItem("Clear Overlay", cbHideOverlay);
buttonConvertToVector = CreatePushButtonItem("Convert To Vector", cbConvToVector);
buttonFilterVector = CreatePushButtonItem("Filter Vector", cbFilterVector);
buttonExit = CreatePushButtonItem("Exit", cbQuit);
buttonConvertToVector.Disabled = 1;
buttonFilterVector.Disabled = 1;
button_row = CreateButtonRow(form, buttonClearOverlay, buttonConvertToVector, buttonFilterVector, buttonExit);
button_row.BottomWidget = form;

drawhisto = CreateIconToggleButton(form, "RVCobjects", "histo", "Show as histogram");
drawhisto.LeftWidget = form;
drawhisto.TopWidget = form;
WidgetAddCallback(drawhisto.ValueChangedCallback, cbRedrawGraph);

frame = CreateFrame(form);
frame.TopWidget = drawhisto;
frame.BottomWidget = button_row;
frame.LeftWidget = form;

da = CreateDrawingArea(frame, 180, sample_width + 2 * sample_space + 2);
WidgetAddCallback(da.ExposeCallback, cbRedrawGraph);
group = DispCreate2DGroup();
layer = GroupQuickAddRasterVar(group, Composite);
overlay = GroupQuickAddRasterVar(group, BiomassRaster);
overlay.NullCellsTransparent = 0;
overlay.UsesTransparency = 1;
overlay.DoBlendMask = 1;

height = NumLins(Composite);
width = NumCols(Composite);
while (height > 700 or width > 1000) {
	height /= 2;
	width /= 2;
	}
view = GroupCreateView(group, form, "view", height, width);
view.form.TopWidget = form;
view.form.LeftWidget = frame;
view.form.RightWidget = form;
view.form.BottomWidget = button_row;
LayerHide(overlay, view);
view.ShowDataTips = 0;
view.ScalePosVisible = 0;
DispAddCallback(view.RestoreCallback, cbRedrawGraph);
tool = ViewCreatePolygonTool(view, "My Tool", "edit_query", "standard");
ToolAddCallback(tool.ActivateCallback, cbToolApply);
ToolAddCallback(tool.PositionSetCallback, cbToolPosSet);
ToolAddCallback(tool.PositionClearedCallback, cbToolPosSet);
ViewAddStandardTools(view);
DialogOpen(form);

gc = CreateGCForDrawingArea(da);
gc.TextStyle.JustifyRight = 1;
# Clear the drawing area
ActivateGC(gc);
SetColorRGB(0,0,0);
black.name = "black";
white.name = "white";
FillRect(0, 0, da.width, da.height);
DrawTextSetFont("stork.of");
DrawTextSetColors(black, white);
DrawTextSetHeightPixels(9);

ViewRedraw(view);
ViewActivateTool(view, tool);
ViewSetMessage(view, "Draw around a field and press \"Apply\"");
tip = CreateToolTip(view.DrawingArea, "Draw around a field\nand press \"Apply\"");
tip.Disabled = 0;
tip.Delay = 3000;	# 3 seconds (3000 ms)

setTip();

DialogWaitForClose(form);

ViewDestroy(view);
GroupDestroy(group);
DestroyWidget(form);
